@@ -2,7 +2,7 @@ package DateTime::Format::Flexible;
 use strict;
 use warnings;
 
-our $VERSION = '0.25';
+our $VERSION = '0.32';
 
 use base 'DateTime::Format::Builder';
 
@@ -11,22 +11,23 @@ use DateTime::Infinite;
 
 use Carp 'croak';
 
-my $DELIM  = qr{(?:\\|\/|-|\.|\s)};
+my $DELIM  = qr{(?:\\|\/|-|'|\.|\s)};
 my $HMSDELIM = qr{(?:\.|:)};
 my $YEAR = qr{(\d{1,4})};
-my $MON = qr{(\d\d?)};
-my $DAY = qr{(\d\d?)};
-my $HOUR = qr{(\d\d?)};
-my $HM = qr{(\d\d?)$HMSDELIM(\d\d?)};
-my $HMS = qr{(\d\d?)$HMSDELIM(\d\d?)$HMSDELIM(\d\d?)};
-my $HMSNS = qr{T?(\d\d?)$HMSDELIM(\d\d?)$HMSDELIM(\d\d?)$HMSDELIM(\d+)T?};
+my $MON = qr{([0-1]?\d)};
+my $DAY = qr{([0-3]?\d)};
+my $HOUR = qr{([0-2]?\d)};
+my $HM = qr{([0-2]?\d)$HMSDELIM([0-5]?\d)};
+my $HMS = qr{([0-2]?\d)$HMSDELIM([0-5]?\d)$HMSDELIM([0-5]?\d)};
+my $HMSNS = qr{T?([0-2]?\d)$HMSDELIM([0-5]?\d)$HMSDELIM([0-5]?\d)$HMSDELIM(\d+)T?};
 my $AMPM = qr{(a\.?m?|p\.?m?)\.?}i;
 
-my $MMDDYYYY = qr{(\d{1,2})$DELIM(\d{1,2})$DELIM(\d{1,4})};
-my $YYYYMMDD = qr{(\d{4})$DELIM(\d{1,2})$DELIM(\d{1,2})};
+my $MMDDYYYY = qr{(\d{1,2})${DELIM}(\d{1,2})${DELIM}(\d{1,4})};
+my $YYYYMMDD = qr{(\d{4})${DELIM}${MON}${DELIM}${DAY}};
+my $YYMMDD = qr{(\d\d)${DELIM}?([0-1]\d)${DELIM}?([0-3]\d)};
 my $MMYY = qr{(\d{1,2})${DELIM}(\d{1,2})}; # YEAR must be > 31 unless MMYY
 my $MMDD = qr{(\d{1,2})$DELIM(\d{1,2})};
-my $XMMXDD = qr{X(\d{1,2})X${DELIM}?(\d{1,2})};
+my $XMMXDD = qr{X([0-1]?\d)X${DELIM}?([0-3]?\d)};
 my $DDXMMX = qr{(\d{1,2})${DELIM}?X(\d{1,2})X};
 my $DDXMMXYYYY = qr{(\d{1,2})${DELIM}X(\d{1,2})X$DELIM(\d{1,4})};
 my $MMYYYY = qr{(\d{1,2})$DELIM(\d{4})};
@@ -60,10 +61,13 @@ my $MDHMS = [ qw( month day hour minute second ) ];
 my $MDHMSAP = [ qw( month day hour minute second ampm ) ];
 my $MYHMS = [ qw( month year hour minute second ) ];
 my $MYHMSAP = [ qw( month year hour minute second ampm ) ];
+my $MDYHM = [ qw( month day year hour minute second ) ];
 my $MDYHMS = [ qw( month day year hour minute second ) ];
 my $MDYHMAP = [ qw( month day year hour minute ampm ) ];
 my $MDYHMSAP = [ qw( month day year hour minute second ampm ) ];
 my $MDHMSY = [ qw( month day hour minute second year ) ];
+my $MDHMSNSY = [ qw( month day hour minute second nanosecond year ) ];
+
 
 my $Y = [ qw( year ) ];
 my $YM = [ qw( year month ) ];
@@ -95,7 +99,6 @@ sub base
 my $formats =
 [
  [ preprocess => \&_fix_alpha ] ,
-
  { length => [18..22] , params => $YMDHMSAP , regex => qr{\A(\d{4})$DELIM(\d{2})$DELIM(\d{2})\s$HMS\s?$AMPM\z} , postprocess => \&_fix_ampm } ,
 
  # 2011-06-16-17.43.30.000000
@@ -106,16 +109,17 @@ my $formats =
  # M/DD/Y, MM/D/Y, M/D/Y, MM/DD/Y, M/D/YY, M/DD/YY, MM/D/Y, MM/SS/YY,
  # M/D/YYYY, M/DD/YYYY, MM/D/YYYY, MM/DD/YYYY
 
- { length => [5..10],  params => $MDY,      regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\z},               postprocess => \&_fix_year },
+ { length => [5..10],  params => $MDY,      regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\z},               postprocess => [ \&_fix_year, \&_fix_zero_month ] },
  { length => [12..14], params => $MDY,      regex => qr{\AX${MON}X${DELIM}n${DAY}n${DELIM}${YEAR}\z} },
- { length => [11..19], params => $MDYHMS,   regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HMS\z},         postprocess => \&_fix_year },
- { length => [11..20], params => $MDYHMAP,  regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HM\s?$AMPM\z},  postprocess => [ \&_fix_ampm , \&_fix_year ] } ,
- { length => [14..22], params => $MDYHMSAP, regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HMS\s?$AMPM\z}, postprocess => [ \&_fix_ampm , \&_fix_year ] } ,
+ { length => [11..19], params => $MDYHMS,   regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HMS\z},         postprocess => [ \&_fix_year, \&_fix_zero_month ] },
+ { length => [11..20], params => $MDYHMAP,  regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HM\s?$AMPM\z},  postprocess => [ \&_fix_ampm , \&_fix_year, \&_fix_zero_month ] } ,
+ { length => [14..22], params => $MDYHMSAP, regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s$HMS\s?$AMPM\z}, postprocess => [ \&_fix_ampm , \&_fix_year, \&_fix_zero_month ] } ,
+
+ # 02/28/2014 14:30 (missing seconds)
+ { length => [14..16], params => $MDYHM,    regex => qr{\A$MMDDYYYY\s$HM\z}, postprocess => [\&_set_default_seconds, \&_fix_zero_month] } ,
 
  ########################################################
  ##### Year/Month/Day
- ##### Can't have 1,2 digit years in this format, would get confused
- ##### with MM-DD-YY
  # YYYY/M/D, YYYY/M/DD, YYYY/MM/D, YYYY/MM/DD
  # YYYY/MM/DD HH:MM:SS
  # YYYY-MM HH:MM:SS
@@ -129,6 +133,10 @@ my $formats =
  { length => [14..19], params => $YMDHMAP,  regex => qr{\A$YYYYMMDD\s$HM\s?$AMPM\z}, postprocess => \&_fix_ampm },
  { length => [14..19], params => $YMDHMS,   regex => qr{\A$YYYYMMDD\s$HMS\z} },
  { length => [17..21], params => $YMDHMSAP, regex => qr{\A$YYYYMMDD\s$HMS\s?$AMPM\z}, postprocess => \&_fix_ampm },
+ # 950404 00:22:12 => 1995-04-04T00:22:12
+ { length => [15],     params => $YMDHMS,   regex => qr{\A$YYMMDD\s$HMS\z}, postprocess => \&_fix_year },
+ { length => [19],     params => $YMDHMSNS, regex => qr{\A$YYMMDD\s$HMSNS\z}, postprocess => \&_fix_year },
+
 
  ########################################################
  ##### YYYY-MM-DDTHH:MM:SS
@@ -161,7 +169,7 @@ my $formats =
  ##### year must be 4 digits unless it is > 31
  ##### or MMYY is true
  # M/YYYY, MM/YYYY
- { length => [6,7], params => $MY, regex => qr{\A$MMYYYY\z} },
+ { length => [6,7], params => $MY, regex => qr{\A$MMYYYY\z}, postprocess => \&_fix_zero_month },
  { length => [3..5], params => $MY, regex => qr{\A$MMYY\z},
    postprocess => [sub {
        my %args = @_;
@@ -184,20 +192,21 @@ my $formats =
  { length => [12..17], params => $MDHMSAP, regex => qr{\A$MMDD\s$HMS\s?$AMPM\z},
    postprocess => [sub { my %args = @_; $args{parsed}{year} = __PACKAGE__->base->year },\&_fix_ampm] },
 
-
  ########################################################
  ##### Dates with at in their name: 12-10-65 at 5:30:25
  # the language plugins should wrap the time like this: T5:30:25T
  # 2005-06-12 T3Tp (15)
- { length => [16],     params => $MDYHMS,   regex => qr{\A${MON}${DAY}${YEAR}T${HMS}T\z},                               postprocess => \&_fix_year } ,
- { length => [17,18],  params => $MDYHMS,   regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s?T${HMS}T\z},            postprocess => \&_fix_year } ,
+ { length => [16,17],    params => $MDYHMS,   regex => qr{\A${MON}${DAY}${YEAR}T${HMS}T\z},                               postprocess => [ \&_fix_year, \&_fix_zero_month ] } ,
+ { length => [17,18,20], params => $MDYHMS,   regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s?T${HMS}T\z},            postprocess => [ \&_fix_year, \&_fix_zero_month ] } ,
+ { length => [17..20], params => $DMYHM,    regex => qr{\A${DAY}${DELIM}X${MON}X${DELIM}${YEAR}\sT${HM}T\z},            postprocess => [ \&_fix_year, \&_fix_zero_month ] } ,
  { length => [20],     params => $MDYHMS,   regex => qr{\AX${MON}X${DELIM}${DAY}${DELIM}${YEAR}\s?T${HMS}T\z},          postprocess => \&_fix_year } ,
+ { length => [21],     params => $MDYHMAP,  regex => qr{\A${MON}${DELIM}${DAY}${DELIM}${YEAR}\s?T${HM}T\s${AMPM}\z},    postprocess => [ \&_fix_ampm, \&_fix_zero_month ] } ,
  { length => [20,21],  params => $YMDHMAP,  regex => qr{\A${YYYYMMDD}\s?T${HM}T\s${AMPM}\z},                            postprocess => \&_fix_ampm } ,
  { length => [21,22],  params => $YMDHMSAP, regex => qr{\A${YEAR}${MON}${DAY}\s?T${HMS}T\s${AMPM}\z},                   postprocess => \&_fix_ampm } ,
  { length => [15],     params => $YMDHAP,   regex => qr{\A${YEAR}${DELIM}${MON}${DELIM}${DAY}\s?T${HOUR}T\s?${AMPM}\z}, postprocess => \&_fix_ampm } ,
  { length => [16..18], params => $YMDHM,    regex => qr{\A${YEAR}${DELIM}${MON}${DELIM}${DAY}\s?T${HM}T\z},             postprocess => \&_fix_year } ,
  { length => [21],     params => $YMDHMS,   regex => qr{\A${YEAR}${DELIM}${MON}${DELIM}${DAY}\s?T${HMS}T\z},            postprocess => \&_fix_year } ,
- { length => [16],     params => $MDYHMS,   regex => qr{\A${MON}${DAY}(\d\d)\s?T${HMS}T\z},                             postprocess => \&_fix_year } ,
+ { length => [16],     params => $MDYHMS,   regex => qr{\A${MON}${DAY}(\d\d)\s?T${HMS}T\z},                             postprocess => [ \&_fix_year, \&_fix_zero_month ] } ,
  { length => [16],     params => $YMDHAP,   regex => qr{\A${YEAR}${DELIM}${MON}${DELIM}${DAY}\s?T${HOUR}T${AMPM}\z},    postprocess => \&_fix_ampm } ,
 
  { length => [15,16],  params => $MDHMS,    regex => qr{\A${MON}${DELIM}${DAY}\s?T${HMS}T\z},
@@ -248,6 +257,10 @@ my $formats =
  { length => [16..25], params => $DMYHMSNS, regex => qr{\A${DDXMMXYYYY}\s${HMSNS}\z},       postprocess => \&_fix_year },
  { length => [14..24], params => $DMYHMSAP, regex => qr{\A${DDXMMXYYYY}\s${HMS}\s?$AMPM\z}, postprocess => [ \&_fix_year , \&_fix_ampm ] },
  { length => [9..15] , params => $HMSMD,    regex => qr{\A${HMS}${XMMXDD}\z},               postprocess => \&_set_default_year },
+
+ # Fri Dec  2 22:56:03.500 GMT+0 1994
+ { length => [24],     params => $MDHMSNSY, regex => qr{\A${XMMXDD}\s${HMSNS}\s${YEAR}\z}, },
+
  { length => [9..15] , params => $HMSDM,    regex => qr{\A${HMS}${DELIM}?${DDXMMX}\z},      postprocess => \&_set_default_year },
  { length => [11..17], params => $HMSMDY,   regex => qr{\A${HMS}${XMMXDDYYYY}\z},           postprocess => \&_fix_year },
  { length => [6..11],  params => $HMMD,     regex => qr{\A${HM}${XMMXDD}\z},                postprocess => \&_set_default_year },
@@ -271,6 +284,8 @@ my $formats =
 
  { length => [7..12],  params => $MDY,      regex => qr{\A$XMMXDDYYYY\z},                 postprocess => \&_fix_year },
  { length => [12..21], params => $MDYHMS,   regex => qr{\A$XMMXDDYYYY\s$HMS\z},           postprocess => \&_fix_year },
+ { length => [14..17], params => $MDYHM,   regex => qr{\A$XMMXDDYYYY\s$HM\z},
+   postprocess => sub { my %args = @_; $args{parsed}{second} = 0; _fix_year(%args) } },
  { length => [14..24], params => $MDYHMSAP, regex => qr{\A$XMMXDDYYYY\s$HMS\s?$AMPM\z},   postprocess => [ \&_fix_year , \&_fix_ampm ] },
 
  # YYYY-mon-D, YYYY-mon-DD, YYYY-mon
@@ -321,9 +336,9 @@ my $formats =
  ########################################################
  ##### bare times
  # HH:MM:SS
- { length => [5..8],
+ { length => [5..10],
    params => [ qw( hour minute second ) ] ,
-   regex => qr{\A$HMS\z} ,
+   regex => qr{\AT?${HMS}T?\z} ,
    postprocess => sub {
        my %args = @_;
        $args{parsed}{year} = __PACKAGE__->base->year;
@@ -331,10 +346,22 @@ my $formats =
        $args{parsed}{day} = __PACKAGE__->base->day;
    }
  },
+ # HH:MM:SS AM
+ { length => [7..13],
+   params => [ qw( hour minute second ampm ) ] ,
+   regex => qr{\AT?${HMS}T?\s?$AMPM\z} ,
+   postprocess => [sub {
+       my %args = @_;
+       $args{parsed}{year} = __PACKAGE__->base->year;
+       $args{parsed}{month} = __PACKAGE__->base->month;
+       $args{parsed}{day} = __PACKAGE__->base->day;
+   }, \&_fix_ampm]
+ },
+
  # HH:MM
- { length => [3..5],
+ { length => [3..7],
    params => [ qw( hour minute ) ] ,
-   regex => qr{\A$HM\z} ,
+   regex => qr{\AT?${HM}T?\z} ,
    postprocess => sub {
        my %args = @_;
        $args{parsed}{year} = __PACKAGE__->base->year;
@@ -366,6 +393,7 @@ my $formats =
    }, \&_fix_ampm ]
  } ,
 
+ ########################################################
  # Day of year
  # 1999345 => 1999, 345th day of year
  { length => [5,7],    params => [ qw( year doy ) ] ,
@@ -546,10 +574,13 @@ sub _parse_timezone
         }
     }
 
-    # search for GMT inside the string
+    # search for GMT/UTC inside the string
     # must be surrounded by spaces
     # 5:30 pm GMT 121065
-    if ( my ( $tz ) = $date =~ m{\s(GMT)\s}mx )
+    # Tue Feb 28 14:30:00 UTC 2014
+    # Fri Dec  2 22:56:03 GMT+0 1994
+
+    if ( my ( $tz ) = $date =~ m{\s(GMT(?:\+0)?|UTC)\s}mx )
     {
         $date =~ s{\Q$tz\E}{};
         $p->{time_zone} = 'UTC';
@@ -583,29 +614,121 @@ sub _parse_timezone
     }
 
     # set any trailing offset timezones
-    if ( my ( $tz ) = $date =~ m{((?:\s+)?\+\d{2,4}|\s+\-\d{4})\.?\z}mx )
+    if ( my ( $tz ) = $date =~ m{(
+                                   (?:\s+)?\+\d{2,4}         # ' +04', '+04'
+                                   |\s+\-\d{4}               # ' -0400'
+                                   |(?:\s+)?[-+]\d{2}:\d{2}  # '-04:00', '+04:00'
+                                 )\.?\z}mx )
     {
-        $date =~ s{\Q$tz\E}{};
+        printf( "# possible timezone (%s) in (%s)\n", $tz, $date) if $ENV{DFF_DEBUG};
+        my $original_tz = $tz;
+        $tz =~ s{:}{};
         # some timezones are 2 digit hours, add the minutes part
         $tz = _clean_whitespace( $tz );
         $tz .= '00' if ( length( $tz ) == 3 );
-        $p->{time_zone} = $tz;
-        return ( $date , $p );
+        if ( _is_valid_tz_offset( $tz ) )
+        {
+            printf( "#  timezone matched (%s)\n" , $tz ) if $ENV{DFF_DEBUG};
+            $date =~ s{\Q$original_tz\E\.?\z}{};
+            $p->{time_zone} = $tz;
+            return ( $date , $p );
+        }
+    }
+
+    if ( length( $date ) > 15 and ($date =~ m{\dT\d} or $date =~ m{\d\s\d}))
+    {
+      # this pattern conflicts with 5-08, 01-02-03, 08-Jan-99, 2006-Dec-08
+      # so we need to check the length and make sure it is long enough to be
+      # a full iso datetime, and that is has a 'T' or ' ' (space) surrounded by digits
+      if ( my ( $tz ) = $date =~ m{(
+                                     (?:\s+)?[-+]\d{2,4}     # '-0800', '-08', ' -08'
+                                   )\.?\z}mx )
+      {
+          printf( "# possible timezone (%s) in (%s)\n", $tz, $date) if $ENV{DFF_DEBUG};
+          my $original_tz = $tz;
+          $tz =~ s{:}{};
+          # some timezones are 2 digit hours, add the minutes part
+          $tz = _clean_whitespace( $tz );
+          $tz .= '00' if ( length( $tz ) == 3 );
+          if ( _is_valid_tz_offset( $tz ) )
+          {
+              $date =~ s{\Q$original_tz\E\.?\z}{};
+              $p->{time_zone} = $tz;
+              return ( $date , $p );
+          }
+      }
     }
 
     # search for positive/negative 4 digit timezones that are inside the string
     # must be surrounded by spaces
     # Mon Apr 05 17:25:35 +0000 2010
-    if ( my ( $tz ) = $date =~ m{\s([\+\-]\d{4})\s}mx )
+    if ( my ( $tz ) = $date =~ m{\s(
+                                   [-+]\d{4}        # Mon Apr 05 17:25:35 +0000 2010
+                                   |[-+]\d{2}:\d{2} # Mon Apr 05 17:25:35 +00:00 2010
+                                 )\s}mx )
     {
-        $date =~ s{\Q$tz\E}{};
-        $p->{time_zone} = $tz;
-        return ( $date , $p );
+        my $original_tz = $tz;
+        $tz =~ s{:}{};
+        if ( _is_valid_tz_offset( $tz ) )
+        {
+            $date =~ s{\Q$original_tz\E}{};
+            $p->{time_zone} = $tz;
+            return ( $date , $p );
+        }
     }
 
     return ( $date , $p );
 }
 
+sub _is_valid_tz_offset
+{
+  my ($offset) = @_;
+
+  # https://en.wikipedia.org/wiki/List_of_UTC_time_offsets
+  my $valid = {
+    '-1200' => 1,
+    '-1100' => 1,
+    '-1000' => 1,
+    '-0930' => 1,
+    '-0900' => 1,
+    '-0800' => 1,
+    '-0700' => 1,
+    '-0600' => 1,
+    '-0500' => 1,
+    '-0400' => 1,
+    '-0330' => 1,
+    '-0300' => 1,
+    '-0200' => 1,
+    '-0100' => 1,
+    '-0000' => 1,
+    '+0000' => 1,
+    '+0100' => 1,
+    '+0200' => 1,
+    '+0300' => 1,
+    '+0330' => 1,
+    '+0400' => 1,
+    '+0430' => 1,
+    '+0500' => 1,
+    '+0530' => 1,
+    '+0545' => 1,
+    '+0600' => 1,
+    '+0630' => 1,
+    '+0700' => 1,
+    '+0800' => 1,
+    '+0845' => 1,
+    '+0900' => 1,
+    '+0930' => 1,
+    '+1000' => 1,
+    '+1030' => 1,
+    '+1100' => 1,
+    '+1200' => 1,
+    '+1245' => 1,
+    '+1300' => 1,
+    '+1400' => 1,
+  };
+  return exists $valid->{$offset};
+}
+
 sub _do_math
 {
     my ( $string ) = @_;
@@ -628,6 +751,7 @@ sub _clean_whitespace
     $string =~ s{\s+\z}{}mx;    # trim back
 
     $string =~ s{\s+}{ }gmx;    # remove extra whitespace from the middle
+    $string =~ s{"}{}gmx;
     return $string;
 }
 
@@ -664,6 +788,30 @@ sub _fix_ampm
     return 1;
 }
 
+sub _fix_zero_month
+{
+    my %args = @_;
+
+    return 1 if not exists $args{parsed}{month};
+    return 1 if not defined $args{parsed}{month};
+
+    if ($args{parsed}{month} == 0)
+    {
+        # they probably meant october
+        print( "#  month => 0 => 10\n") if $ENV{DFF_DEBUG};
+        $args{parsed}{month} = 10;
+    }
+
+    return 1;
+}
+
+sub _set_default_seconds
+{
+    my %args = @_;
+    $args{parsed}{second} = 0;
+    return 1;
+}
+
 sub _set_default_year
 {
     my %args = @_;
@@ -994,7 +1142,7 @@ Tom Heady <cpan@punch.net>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2007-2012 Tom Heady.
+Copyright 2007-2018 Tom Heady.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of either:
